#!/bin/sh
# Copyright (c) 2000-2018 Synology Inc. All rights reserved.

PACKAGE_NAME=${PACKAGE_NAME:-"SafeAccess"}
PACKAGE_TARGET_DIR="/var/packages/${PACKAGE_NAME}/target"
PACKAGE_WEBAPI_DIR="${PACKAGE_TARGET_DIR}/webapi"
PACKAGE_BIN_DIR="${PACKAGE_TARGET_DIR}/bin"
PACKAGE_CONF_DIR="/usr/syno/etc/packages/${PACKAGE_NAME}"

UPSTART_DIR="/etc/init"
DATABASE_UPDATE_HOOK_DIR="/usr/libexec/database_autoupdate/version_change"
TOPOLOGY_HOOK_DIR="/usr/libexec/net/topology_change/"

install_project()
{
	ipset create AC_DROP_IFACES hash:net,iface
	ipset create AC_DROP_MACS hash:mac
	ipset create AC_RETURN_MACS hash:mac
	ipset create AC_BAN_MACS hash:mac
	iptables -t filter -A ACCESS_BAN_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	iptables -t filter -A ACCESS_BAN_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	iptables -t filter -A ACCESS_BAN_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	iptables -t filter -A ACCESS_BAN_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN
	iptables -t filter -A ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS src -j DROP
	iptables -t filter -A ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS dst -j DROP
	iptables -t filter -A ACCESS_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	iptables -t filter -A ACCESS_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	iptables -t filter -A ACCESS_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	iptables -t filter -A ACCESS_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN
	iptables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS src -j RETURN
	iptables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS dst -j RETURN
	iptables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS src -j DROP
	iptables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS dst -j DROP
	iptables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES src -j DROP
	iptables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES dst -j DROP

	ip6tables -t filter -A ACCESS_BAN_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	ip6tables -t filter -A ACCESS_BAN_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	ip6tables -t filter -A ACCESS_BAN_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	ip6tables -t filter -A ACCESS_BAN_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN
	ip6tables -t filter -A ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS src -j DROP
	ip6tables -t filter -A ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS dst -j DROP
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS src -j RETURN
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS dst -j RETURN
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS src -j DROP
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS dst -j DROP
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES src -j DROP
	ip6tables -t filter -A ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES dst -j DROP

	mkdir -p ${DATABASE_UPDATE_HOOK_DIR}
	${PACKAGE_BIN_DIR}/synoactool --init-database &
	${PACKAGE_BIN_DIR}/synoac_general_trigger --enable
	${PACKAGE_BIN_DIR}/synoac_clean_db_trigger --enable
	ln -sf ${PACKAGE_BIN_DIR}/synoac_update_db_hook ${DATABASE_UPDATE_HOOK_DIR}/synoac_update_db_hook
	ln -sf ${PACKAGE_BIN_DIR}/topology_hook.sh ${TOPOLOGY_HOOK_DIR}/accesscontrol.sh
	cp ${PACKAGE_TARGET_DIR}/etc/synoaccesscontrol/synotimespentd.upstart ${UPSTART_DIR}/synotimespentd.conf
	cp ${PACKAGE_TARGET_DIR}/etc/synoaccesscontrol/synoac_reload_ban_status.upstart ${UPSTART_DIR}/synoac_reload_ban_status.conf
}

uninstall_project()
{
	iptables -t filter -D ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS src -j DROP
	iptables -t filter -D ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS dst -j DROP
	iptables -t filter -D ACCESS_BAN_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	iptables -t filter -D ACCESS_BAN_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	iptables -t filter -D ACCESS_BAN_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	iptables -t filter -D ACCESS_BAN_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN
	iptables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS src -j RETURN
	iptables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS dst -j RETURN
	iptables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS src -j DROP
	iptables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS dst -j DROP
	iptables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES src -j DROP
	iptables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES dst -j DROP
	iptables -t filter -D ACCESS_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	iptables -t filter -D ACCESS_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	iptables -t filter -D ACCESS_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	iptables -t filter -D ACCESS_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN

	ip6tables -t filter -D ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS src -j DROP
	ip6tables -t filter -D ACCESS_BAN_FILTER_FORWARD -m set --match-set AC_BAN_MACS dst -j DROP
	ip6tables -t filter -D ACCESS_BAN_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	ip6tables -t filter -D ACCESS_BAN_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	ip6tables -t filter -D ACCESS_BAN_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	ip6tables -t filter -D ACCESS_BAN_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS src -j RETURN
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_RETURN_MACS dst -j RETURN
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS src -j DROP
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_MACS dst -j DROP
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES src -j DROP
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -m set --match-set AC_DROP_IFACES dst -j DROP
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -i lbr0 -o gbr0 -j RETURN
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -i gbr0 -o lbr0 -j RETURN
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -i gbr0 -o gbr0 -j RETURN
	ip6tables -t filter -D ACCESS_FILTER_FORWARD -i lbr0 -o lbr0 -j RETURN

	ipset destroy AC_DROP_IFACES
	ipset destroy AC_DROP_MACS
	ipset destroy AC_RETURN_MACS
	ipset destroy AC_BAN_MACS

	${PACKAGE_BIN_DIR}/synoac_general_trigger --disable
	${PACKAGE_BIN_DIR}/synoac_clean_db_trigger --disable
	rm -f ${DATABASE_UPDATE_HOOK_DIR}/synoac_update_db_hook
	rm -f ${TOPOLOGY_HOOK_DIR}/accesscontrol.sh
	rm -f ${UPSTART_DIR}/synotimespentd.conf
	rm -f ${UPSTART_DIR}/synoac_reload_ban_status.conf
}

start_project()
{
	${PACKAGE_BIN_DIR}/synoac_general_trigger --refresh-ban-config-group
	start synotimespentd
}

stop_project()
{
	stop synotimespentd
}

case $1 in
	--install)
		install_project
		exit 0
	;;
	--uninstall)
		uninstall_project
		exit 0
	;;
	--start)
		start_project
		exit 0
	;;
	--stop)
		stop_project
		exit 0
	;;
	--status)
		exit 0
	;;
	--killall)
        ;;
	--log)
		exit 0
	;;
esac

exit 0
